#! /usr/bin/env python
# -*- coding: utf-8 -*-

import json
import hashlib
import time
import socket
import sys
import uuid
import base64
import mimetypes
import os

reload(sys)
sys.setdefaultencoding("utf-8")

PY3 = sys.version_info[0] == 3

if PY3:
    import http.client as httplib
    from urllib.parse import quote, urlencode
else:
    import httplib
    from urllib import quote, urlencode


class MTCloud():

    openID = ''
    openToken = ''

    endpoint = 'api.talk-fun.com'
    endpoint2 = 'api-1.talk-fun.com'

    version = '1.3'

    format = 'json'

    timeout = 10

    debug = False

    CODE_FAIL = -1
    CODE_SUCCESS = 0
    CODE_PARAM_ERROR = 10
    CODE_VIDEO_UPLOADED = 1281
    CODE_SIGN_EXPIRE = 10002
    CODE_SIGN_ERROR = 10003

    ROLE_USER = 'user'
    ROLE_ADMIN = 'admin'
    ROLE_SPADMIN = 'spadmin'
    ROLE_GUEST = 'guest'

    USER_GENDER_UNKNOW = 0
    USER_GENDER_MALE = 1
    USER_GENDER_FEMALE = 2

    ACCOUNT_TYPE_MT    = 1
    ACCOUNT_TYPE_THIRD = 2

    LIVE_NO_PLAYBACK = 0
    LIVE_HAS_PLAYBACK = 1

    VOICE_FLOW_CLOUD = 1
    VOICE_FLOW_LISTEN_ONLY = 2
    VOICE_FLOW_AUTO = 3

    CUSTOM_RTMP_RATIO_4_3 = 1;
    CUSTOM_RTMP_RATIO_16_9 = 2;

    def __init__(self,openID=None,openToken=None):

        if openID is not None:
            self.openID = openID.strip()

        if openToken is not None:
            self.openToken = openToken.strip()

    def userAccess(self,uid,nickname,role,roomid,expire=3600,options=[]):
        params = {
                'uid':uid,
                'nickname':nickname,
                'role':role,
                'roomid':roomid,
                'expire':expire,
                'options':options
                }
        return self.call('user.access',params)

    def userAccessPlayback(self,uid,nickname,role,liveid,expire,option = []):
        params = {'uid':uid,'nickname':nickname,'role':role,'liveid':liveid,'expire':expire,'option':option}
        return self.call('user.access.playback', params)

    def userAccessUrl(self,uid,nickname,role,roomid,expire=3600,options=[]):
        accessAuth = self.userAccessKey(uid, nickname, role, roomid, expire, options)
        return 'http://open.talk-fun.com/room.php?accessAuth=' + accessAuth

    def userAccessKey(self,uid,nickname,role,roomid,expire=3600,options=[]):
        params = {'openID': self.openID.strip(), 'timestamp': str(time.time()), 'uid': uid, 'nickname': nickname,
                  'role': role, 'roomid': roomid, 'expire': expire, 'options': quote(json.dumps(options))}

        params['sign'] = self.generateSign(params)
        accessAuth = self.base64UrlEncode(quote(json.dumps(params)))
        return accessAuth

    def userAccessPlaybackUrl(self, uid, nickname, role, liveid, expire, options=[]):
        accessAuth = self.userAccessPlaybackKey(uid, nickname, role, liveid, expire, options)
        return 'http://open.talk-fun.com/player.php?accessAuth=' + accessAuth

    def userAccessPlaybackKey(self, uid, nickname, role, liveid, expire, options=[]):
        params = {'openID': self.openID.strip(), 'timestamp': str(time.time()), 'uid': uid, 'nickname': nickname,
                  'role': role, 'liveid': liveid, 'expire': expire, 'options': quote(json.dumps(options))}

        params['sign'] = self.generateSign(params)
        accessAuth = self.base64UrlEncode(quote(json.dumps(params)))
        return accessAuth

    def userAccessPlaybackAlbum(self,uid,nickname,role,albumId,expire,options = []):
        params = {'uid':uid,'nickname':nickname,'role':role,'album_id':albumId,'expire':expire,'option':options}
        return self.call('user.access.playbackAlbum', params)

    def userAccessPlaybackAlbumUrl(self,uid,nickname,role,albumId,expire,options = []):
        accessAuth = self.userAccessPlaybackAlbumKey(uid, nickname, role, albumId, expire, options)
        return 'http://open.talk-fun.com/player.php?accessAuth=' + accessAuth

    def userAccessPlaybackAlbumKey(self,uid,nickname,role,albumId,expire,options = []):
        params = {'openID': self.openID.strip(), 'timestamp': str(time.time()), 'uid': uid, 'nickname': nickname,
                  'role': role, 'album_id': albumId, 'expire': expire, 'options': quote(json.dumps(options))}

        params['sign'] = self.generateSign(params)
        accessAuth = self.base64UrlEncode(quote(json.dumps(params)))
        return accessAuth

    def userAccessPlaybackClip(self,uid,nickname,role,clipid,expire,options = []):
        params = {'uid': uid, 'nickname': nickname, 'role': role, 'clipid': clipid, 'expire': expire,
                  'option': options}
        return self.call('user.access.playbackClip', params)

    def userAccessPlaybackClipUrl(self,uid,nickname,role,clipid,expire,options = []):
        accessAuth = self.userAccessPlaybackClipKey(uid, nickname, role, clipid, expire, options)
        return 'http://open.talk-fun.com/player.php?accessAuth=' + accessAuth

    def userAccessPlaybackClipKey(self,uid,nickname,role,clipid,expire,options = []):
        params = {'openID': self.openID.strip(), 'timestamp': str(time.time()), 'uid': uid, 'nickname': nickname,
                  'role': role, 'clipid': clipid, 'expire': expire, 'options': quote(json.dumps(options))}

        params['sign'] = self.generateSign(params)
        accessAuth = self.base64UrlEncode(quote(json.dumps(params)))
        return accessAuth

    def userOnlineList(self,roomid,start_time,end_time):
        params = {'roomid':roomid,'start_time':start_time,'end_time':end_time}
        return self.call('user.online.list',params)

    def roomGetInfo(self,roomid):
        params = {'roomid': roomid}
        return self.call('room.getInfo',params)

    def roomGetUrl(self,roomid):
        params = {'roomid': roomid}
        return self.call('room.getUrl',params)

    def roomCreate(self,roomName,voiceFlow = None, authKey = ''):
        if voiceFlow is None:
            voiceFlow = self.VOICE_FLOW_CLOUD
        params = {'roomName': roomName,'authKey':authKey,'voiceFlow':voiceFlow}
        return self.call('room.create',params)

    def roomAutoCreate(self,userUnique,nickname):
        params = {'userUnique': userUnique,'nickname': nickname}
        return self.call('room.autocreate',params)

    def roomCreatev2(self, roomName, authKey = '', userKey = '', zhuboKey = '', modetype = 3, options = []):
        params = {'roomName': roomName,'authKey':authKey,'userKey':userKey,'zhuboKey':zhuboKey,'modetype':modetype,'options':options}
        return self.call('room.createv2',params)

    def roomUpdate(self,roomid,roomName):
        params = {'roomid':roomid,'roomName':roomName}
        return self.call('room.update',params)

    def roomDrop(self,roomid):
        params = {'roomid': roomid}
        return self.call('room.drop',params)

    def roomList(self,page=1,size=20):
        params = {'page': page,'size': size}
        return self.call('room.list',params)

    def roomBindAccount(self,roomid,account,accountType = None):
        if accountType is None:
            accountType = self.ACCOUNT_TYPE_MT
        params = {'roomid': roomid,'account': account,'accountType': accountType}
        return self.call('room.live.bindAccount',params)

    def roomUnbindAccount(self,roomid,account,accountType = None):
        if accountType is None:
            accountType = self.ACCOUNT_TYPE_MT
        params = {'roomid': roomid,'account': account,'accountType': accountType}
        return self.call('room.live.unbindAccount',params)

    def roomBroadcastSend(self, roomid, cmd, args=[], options=[]):
        params = {'roomid': roomid,'cmd': cmd,'params':args,'options': options}
        return self.call('room.broadcast.send', params);

    def roomAddRobot(self, roomid, userList, total = 0):
        params = {'roomid': roomid, 'userList': userList, 'total': total}
        return self.call('room.robot.add', params, 'POST')

    def roomNoticeRoll(self, roomid, content, link, duration):
        params = {'roomid': roomid,'content': content,'link': link,'duration': duration}
        return self.call('room.notice.roll', params)

    def roomPushRtmpUrl(self, roomid, title, ratio):
        params = {'roomid': roomid,'title': title,'ratio': ratio}
        return self.call('room.pushRtmpUrl', params)

    def roomLogin(self, roomid, options = []):
        params = {'roomid': roomid, 'options': options}
        return self.call('room.login', params)

    def zhuboLogin(self,account,accountType,options):
        params = {'account':account,'accountType':accountType,'options':options}
        return self.call('zhubo.login', params)

    def zhuboUpdatePortrait(self,account,accountType, filename):
        params = {'account': account, 'accountType': accountType}
        ret = self.call('zhubo.portrait.uploadurl', params)
        ret = eval(ret)

        if ret['code'] == self.CODE_SUCCESS:
            file = {'field': ret['data']['field'], 'file': filename, 'name': 'portrait.jpg'}
            return self.uploadFile(ret['data']['api'], params, file)

        return {'code': self.CODE_FAIL, 'msg': '该主播不存在'}

    def zhuboRoomLogin(self,roomid,options):
        params = {'roomid': roomid,'options':options}
        return self.call('zhubo.room.login', params);

    def zhuboGet(self,account,accountType = None):
        if accountType is None:
            accountType = self.ACCOUNT_TYPE_MT
        params = {'account': account,'accountType': accountType}
        return self.call('zhubo.get',params)

    def zhuboCreate(self,account,nickname,accountType = None,password = None,intro = '',departmentID=0):
        if accountType is None:
            accountType = self.ACCOUNT_TYPE_MT
        if password is None:
            password = ''
        params = {'account': account,'nickname': nickname,'accountType': accountType,'password': password,'departmentID':departmentID}
        return self.call('zhubo.create',params)

    def zhuboUpdateInfo(self,account,accountType,nickname,intro = '',departmentID=0):
        params = {'account':account,'accountType':accountType,'nickname':nickname,'intro':intro,'departmentID':departmentID}
        return self.call('zhubo.update.info',params)

    def zhuboUpdatePassword(self,account,accountType,password):
        params = {'account': account,'accountType': accountType,'password': password}
        return self.call('zhubo.update.password',params)

    def zhuboDel(self,account,accountType = None):
        if accountType is None:
            accountType = self.ACCOUNT_TYPE_MT
        params = {'account': account,'accountType': accountType}
        return self.call('zhubo.del',params)

    def zhuboList(self,page = 1,size = 10):
        params = {'page': page,'size': size}
        return self.call('zhubo.list',params)

    def liveGet(self,liveid,expire = 3600,visitNumberLimit = 0,check_ip = 0,uid = ''):
        params = {'liveid': liveid,'expire': expire,'numberLimit': visitNumberLimit,'c_ip': check_ip,'uid': uid};
        return self.call('live.get',params)

    def liveGetBatch(self,liveids,expire = 3600):
        params = {'liveids': liveids,'expire': expire}
        return self.call('live.getBatch',params)

    def liveGetLast(self,size = 1,roomid = 0):
        params = {'size': size, 'roomid': roomid}
        return self.call('live.getlast', params);

    def liveList(self,startDate,endDate,page = 1,size = 10,playback = 0):
        params = {'startDate': startDate,'endDate': endDate,'page': page,'size': size,'playback': playback}
        return self.call('live.list',params)

    def liveListAll(self,page = 1,size = 10,order = 'desc', roomid = 0):
        params = {'page': page,'size': size,'order': order,'roomid': roomid}
        return self.call('live.listall', params);

    def liveMessageList(self,liveid,page = 1):
        params = {'liveid': liveid, 'page': page}
        return self.call('live.message', params);

    def liveFlowerList(self,liveid,page = 1,size = 10):
        params = {'liveid': liveid, 'page': page, 'size': size}
        return self.call('live.flower.list', params);

    def albumCreate(self,album_name,liveids = []):
        params = {'album_name': album_name, 'liveids': liveids}
        return self.call('album.create', params)

    def albumGet(self,album_id,expire = 3600):
        params = {'album_id': album_id, 'expire': expire}
        return self.call('album.get', params)

    def albumDelete(self,album_id):
        params = {'album_id': album_id}
        return self.call('album.delete', params)

    def albumAdd(self,album_id,liveids = []):
        params = {'album_id': album_id, 'liveids': liveids}
        return self.call('album.add', params)

    def albumRemove(self,album_id,liveids = []):
        params = {'album_id': album_id, 'liveids': liveids}
        return self.call('album.remove', params)

    def albumCreateCourse(self,album_name,course_ids = []):
        params = {'album_name': album_name, 'course_ids': course_ids}
        return self.call('album.course.create', params)

    def albumAddCourse(self,album_id,course_ids = []):
        params = {'album_id': album_id, 'course_ids': course_ids}
        return self.call('album.course.add', params)

    def albumRemoveCourse(self,album_id,course_ids = []):
        params = {'album_id': album_id, 'course_ids': course_ids}
        return self.call('album.course.remove', params)

    def courseConfig(self,course_id):
        params = {'course_id':course_id}
        return self.call('course.getConfig', params)

    def courseUpdateConfig(self,course_id, options = []):
        params = {'course_id': course_id, 'options': options}
        return self.call('course.updateConfig', params)

    def courseRobotSet(self,course_id, userList = [], total = 0):
        params = {'course_id': course_id, 'userList': userList, 'total': total}
        return self.call('course.robot.set', params, 'POST')

    def courseUploadPlayback(self,course_id, account, accountType, file):
        params = {'account':account,'accountType':accountType,'course_id':course_id}
        res = self.call('course.playback.uploadUrl', params)
        res = eval(res)

        if res['code'] == MTCloud.CODE_SUCCESS and res['data']:
            params = res['data']['params']

            file['field'] = res['data']['field']
            return self.uploadFile(res['data']['api'], params, file)

        return res;

    def liveRoomGet(self,roomid,start_time,expire = 3600):
        params = {'roomid': roomid,'start_time': start_time,'expire': expire}
        return self.call('live.room.get',params)

    def liveRoomList(self,roomid,start_time,end_time,expire):
        params = {'roomid':roomid,'start_time':start_time,'end_time':end_time,'expire':expire}
        return self.call('live.room.list',params)

    def liveVisitorList(self,liveid,page = 1,size = 10):
        params = {'liveid': liveid, 'page': page, 'size': size}
        return self.call('live.visitor.list',params)

    def liveVisitorGet(self,liveid, uid):
        params = {'liveid': liveid,'uid': uid}
        return self.call('live.visitor.get',params)

    def liveQuestionList(self,liveid,page = 1,size = 10):
        params = {'liveid': liveid,'page': page,'size': size}
        return self.call('live.question.list',params)

    def liveAudioDownloadUrl(self,liveid):
        params = {'liveid': liveid}
        return self.call('live.audio.download.url',params)

    def livePlaybackVisitorList(self,liveid,page = 1,size = 10):
        params = {'liveid': liveid,'page': page,'size': size}
        return self.call('live.playback.visitor.list',params)

    def livePlaybackVisitorTimeList(self,start_time,end_time,page = 1,size = 10):
        params = {'start_time': start_time,'end_time': end_time,'page': page,'size': size}
        return self.call('live.playback.visitor.timelist',params)

    def livePlaybackVideo(self,liveid):
        params = {'liveid': liveid}
        return self.call('live.playback.video',params)

    def livePlaybackLoginUrl(self,liveid):
        params = {'liveid': liveid}
        return self.call('live.playback.loginUrl',params)

    def liveVoteList(self,liveid, page = 1, size = 10):
        params = {'liveid': liveid,'page': page,'size': size}
        return self.call('live.vote.list',params)

    def liveVoteDetail(self,vid, liveid, page = 1, size = 10):
        params = {'vid': vid,'liveid': liveid,'page': page,'size': size}
        return self.call('live.vote.detail',params)

    def liveLotteryList(self,liveid, page = 1, size = 10):
        params = {'liveid': liveid,'page': page,'size': size}
        return self.call('live.lottery.list',params)

    def liveVoteAdd(self,roomid, uid, nickname, title, label, op, type, optional, answer = '', image = '', options = {}):
        params = {'roomid':roomid,'uid':uid,'nickname':nickname,'title':title,'label':label,'op':op,'type':type,'optional':optional,'answer':answer,'options':options}

        file = ''

        if ''!=image:
            file = {'field':'image', 'file':image, 'name':'image.jpg'}

        return self.call('live.vote.add',params,'POST',file)

    def liveVoteEnd(self,vid,showResult,uid,nickname):
        params = {'vid':vid,'showResult':showResult,'uid':uid,'nickname':nickname}

        return self.call('live.vote.end',params,'POST')

    def liveVoteEmit(self, vid, roomid):
        params = {'vid':vid, 'roomid':roomid}

        return self.call('live.vote.emit', params)

    def liveVoteDelete(self, vid):
        params = {'vid':vid}

        return self.call('live.vote.delete', params)

    def liveVoteUpdate(self, vid, options):
        params = {'vid':vid, 'options':options}

        if (options.has_key('image')):
            file = ''

            if (os.path.isfile(options['image'])):
                file = {'field': 'image', 'file': options['image'], 'name': 'image.jpg'}
            else:
                ret = {'code':self.CODE_FAIL, 'msg':'文件' + options['image'] + '不存在'}
                return json.dumps(ret)

            return self.call('live.vote.update', params, 'POST', file)
        else:
            return self.call('live.vote.update', params)

    def liveChapterList(self, liveid):
        params = {'liveid':liveid}

        return self.call('live.chapter.list',params,'POST')

    def liveUploadPlayback(self, account, accountType, file, title = ''):
        params = {'account':account,'accountType':accountType,'title':title}
        res = self.call('live.playback.uploadUrl', params)
        res = eval(res)

        if res['code'] == MTCloud.CODE_SUCCESS and res['data']:
            params = res['data']['params']

            file['field'] = res['data']['field']
            self.timeout = 3600

            return self.uploadFile(res['data']['api'], params, file)

        return res

    def liveQaAdd(self, roomid, content, uid, role, nickname, options = {}):
        params = {'roomid':roomid, 'content':content, 'uid':uid, 'role':role, 'nickname':nickname, 'options':options}
        return self.call('live.qa.add', params)

    def liveQaAudit(self, qid, roomid):
        params = {'qid':qid, 'roomid':roomid}
        return self.call('live.qa.audit', params)

    def liveQaDelete(self, qid, roomid):
        params = {'qid':qid, 'roomid':roomid}
        return self.call('live.qa.delete', params)

    def liveQaAnswer(self, qid, roomid, content, uid, nickname, options = {}):
        params = {'qid':qid, 'roomid':roomid, 'content':content, 'uid':uid, 'nickname':nickname, 'options':options}
        return self.call('live.qa.answer', params)

    def liveQaList(self, roomid, options = {}):
        params = {'roomid':roomid, 'options':options}
        return self.call('live.qa.list', params)

    def liveQaTotal(self, liveid):
        params = {'liveid':liveid}
        return self.call('live.qa.total', params)

    def livePrivateChatList(self, liveid, page = 1, size = 10):
        params = {'liveid': liveid,'page': page,'size': size}
        return self.call('live.privateChat', params)

    def courseAdd(self,course_name,account,start_time,end_time,nickname = None,accountIntro = None,options = {}):
        if nickname is None:
            nickname = account

        params = {'course_name': course_name,'account': account,'start_time': start_time,'end_time': end_time,'nickname': nickname,'accountIntro': accountIntro,'options': options}
        return self.call('course.add',params)

    def courseAccess(self,course_id,uid,nickname,role,expire = 3600,options = []):
        params = {'course_id': course_id,'uid': uid,'nickname': nickname,'role': role,'expire': expire,'options': options}
        return self.call('course.access',params)

    def courseAccessPlayback(self,course_id,uid,nickname,role,expire = 3600,options = []):
        params = {'course_id':course_id,'uid':uid,'nickname':nickname,'role':role,'expire':expire,'options':options}
        return self.call('course.access.playback',params)

    def courseAccessUrl(self, course_id, uid, nickname, role, expire = 3600, options=[]):
        accessAuth = self.courseAccessKey(course_id, uid, nickname, role, expire, options)
        return 'http://open.talk-fun.com/room.php?accessAuth=' + accessAuth

    def courseAccessKey(self, course_id, uid, nickname, role, expire = 3600, options=[]):
        params = {'openID': self.openID.strip(), 'timestamp': str(time.time()), 'course_id': course_id, 'uid': uid,
                  'nickname': nickname, 'role': role, 'expire': expire, 'options': quote(json.dumps(options))}
        params['sign'] = self.generateSign(params)
        accessAuth = self.base64UrlEncode(quote(json.dumps(params)))
        return accessAuth

    def courseAccessPlaybackUrl(self, course_id, uid, nickname, role, expire=3600, options=[]):
        accessAuth = self.courseAccessKey(course_id, uid, nickname, role, expire, options)
        return 'http://open.talk-fun.com/player.php?accessAuth=' + accessAuth

    def courseGet(self,course_id):
        params = {'course_id': course_id}
        return self.call('course.get',params)

    def courseDelete(self,course_id):
        params = {'course_id': course_id}
        return self.call('course.delete',params)

    def courseList(self,start_time,end_time,page = 1, size = 10, options={}):
        params = {'start_time': start_time,'end_time': end_time,'page': page, 'size': size, 'options': options}
        return self.call('course.list',params)

    def courseUpdate(self,course_id,account,course_name,start_time,end_time,nickname,accountIntro=''):
        params = {'course_id': course_id,'course_name': course_name,'account': account,'start_time': start_time,'end_time': end_time,'nickname':nickname,'accountIntro':accountIntro}
        return self.call('course.update',params)

    def courseVoteDetail(self,vid, course_id, page = 1, size = 10):
        params = {'vid': vid,'course_id': course_id,'page': page,'size': size}
        return self.call('course.votes.detail',params)

    def courseVoteList(self,course_id, page = 1, size = 10, status = 'normal'):
        params = {'course_id': course_id,'page': page,'size': size,'status': status}
        return self.call('course.votes',params)

    def courseVoteDelete(self, vid):
        params = {'vid':vid}
        return self.call('course.votes.delete', params)

    def courseVoteUpdate(self, vid, options):
        params = {'vid': vid, 'options': options}

        if (options.has_key('image')):
            file = ''

            if (os.path.isfile(options['image'])):
                file = {'field': 'image', 'file': options['image'], 'name': 'image.jpg'}
            else:
                ret = {'code': self.CODE_FAIL, 'msg': '文件' + options['image'] + '不存在'}
                return json.dumps(ret)

            return self.call('course.votes.update', params, 'POST', file)
        else:
            return self.call('course.votes.update', params)

    def courseLotteryList(self,course_id, page = 1, size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('course.lottery.list',params)

    def courseAudioDownloadUrl(self,course_id):
        params = {'course_id': course_id}
        return self.call('course.audio.download.url',params)

    def courseOnlineList(self, course_id, start_time, end_time, page = 1, size = 10):
        params = {'course_id': course_id, 'start_time': start_time, 'end_time': end_time, 'page': page, 'size': size}
        return self.call('course.online.list', params)

    def courseOnlineAdmin(self, start_time, end_time, page = 1, size = 10):
        params = {'start_time': start_time, 'end_time': end_time, 'page': page, 'size': size}
        return self.call('course.online.admin', params)

    def courseVisitorList(self,course_id,page = 1,size = 10,options = []):
        params = {'course_id': course_id,'page': page,'size': size,'options':options}
        return self.call('course.visitor.list',params)

    def coursePlaybackVisitorList(self,course_id,page = 1,size = 10,options = []):
        params = {'course_id': course_id, 'page': page, 'size': size, 'options': options}
        return self.call('course.visitor.playback',params)

    def courseVisitorListAll(self,start_time,end_time,page = 1,size=10):
        params = {'start_time':start_time,'end_time':end_time,'page':page,'size':size}
        return self.call('course.visitor.listall',params)

    def courseQuestionList(self,course_id,page = 1,size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('course.question.list',params)

    def courseFlowerList(self,course_id,page = 1,size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('course.flower.list',params)

    def courseMessageList(self,course_id,page = 1):
        params = {'course_id': course_id,'page': page}
        return self.call('course.message',params)

    def courseDocumentList(self,course_id,page = 1):
        params = {'course_id': course_id,'page': page}
        return self.call('course.document',params)

    def courseDocumentUpload(self, course_id, file):
        params = {'course_id': course_id, 'name': file['name']}

        retval = self.call('course.document.uploadurl.get', params)

        retval = eval(retval)

        if retval['code'] == MTCloud.CODE_SUCCESS and retval['data']['api']:
            file['field'] = retval['data']['field']
            return self.uploadFile(retval['data']['api'], {}, file)

        return retval

    def courseDocumentDelete(self, id):
        params = {'id':id}
        return self.call('document.delete', params)

    def courseVideo(self,course_id):
        params = {'course_id': course_id}
        return self.call('course.video',params)

    def courseNoticeRoll(self,course_id,content,link,duration):
        params = {'course_id':course_id,'content':content,'link':link,'duration':duration}
        return self.call('course.notice.roll',params)

    def courseVoteAdd(self,course_id,uid,nickname,title,label,op,type,optional,answer='',image='',options={}):
        params = {'course_id':course_id,'uid':uid,'nickname':nickname,'title':title,'label':label,'op':op,'type':type,'optional':optional,'answer':answer,'options':options}

        file = ''

        if ''!=image:
            file = {'field':'image', 'file':image, 'name':'image.jpg'}

        return self.call('course.votes.add',params,'POST',file)

    def courseVoteEnd(self,vid,showResult,uid,nickname):
        params = {'vid':vid,'showResult':showResult,'uid':uid,'nickname':nickname}
        return self.call('course.votes.end',params,'POST')

    def courseVoteEmit(self,vid,course_id):
        params = {'vid':vid,'course_id':course_id}
        return self.call('course.votes.emit',params)

    def courseChapterList(self, course_id):
        params = {'course_id':course_id}

        return self.call('course.chapter.list',params,'POST')

    def courseUploadPlayback(self, course_id, file):
        params = {'course_id':course_id}
        res = self.call('course.playback.uploadUrl', params)
        res = eval(res)

        if res['code'] == MTCloud.CODE_SUCCESS and res['data']:
            params = res['data']['params']

            file['field'] = res['data']['field']
            self.timeout = 3600
            return self.uploadFile(res['data']['api'], params, file)

        return res

    def courseBroadcastSend(self, course_id, cmd, args, options = []):
        params = {'course_id': course_id, 'cmd': cmd, 'params': args, 'options': options}
        return self.call('course.broadcast.send', params)

    def courseLogin(self,account,accountType,options):
        params = {'account': account, 'accountType': accountType, 'options': options}
        return self.call('course.login', params)

    def courseQaAdd(self, course_id, content, uid, role, nickname, options = []):
        params = {'course_id': course_id, 'content': content, 'uid': uid, 'role': role, 'nickname': nickname, 'options': options}
        return self.call('course.qa.add', params)

    def courseQaAudit(self, qid, course_id):
        params = {'qid': qid, 'course_id': course_id}
        return self.call('course.qa.audit', params)

    def courseQaDelete(self, qid, course_id):
        params = {'qid': qid, 'course_id': course_id}
        return self.call('course.qa.delete', params)

    def courseQaAnswer(self, qid, course_id, content, uid, nickname, options = []):
        params = {'qid': qid, 'course_id': course_id, 'content': content, 'uid': uid, 'nickname':nickname, 'options': options}
        return self.call('course.qa.answer', params)

    def courseQaList(self, course_id, options = []):
        params = {'course_id': course_id, 'options': options}
        return self.call('course.qa.list', params)

    def courseQaTotal(self, course_id):
        params = {'course_id':course_id}
        return self.call('course.qa.total', params)

    def coursePrivateChatList(self, course_id, page = 1, size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('course.privateChat', params)

    def courseChatDisableList(self, course_id):
        params = {'course_id': course_id}
        return self.call('course.chat.disable.list', params)

    def clipAdd(self,liveid, name, time, isRelated=''):
        params = {'liveid': liveid,'name': name,'time': time,'isRelated': isRelated}
        return self.call('clip.add',params)

    def clipUpdate(self,clipid, name, time, isRelated=''):
        params = {'clipid': clipid,'name': name,'time': time,'isRelated': isRelated}
        return self.call('clip.update',params)

    def clipDelete(self,clipid):
        params = {'clipid': clipid}
        return self.call('clip.delete',params)

    def clipGet(self,clipid):
        params = {'clipid': clipid}
        return self.call('clip.get',params)

    def clipList(self,page = 1, size = 10, liveid = ''):
        params = {'page': page,'size': size,'liveid': liveid}
        return self.call('clip.list',params)

    def clipListByCid(self,course_id,page = 1, size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('clip.course.list',params)

    def clipAddByCid(self,course_id, name, time, isRelated=''):
        params = {'course_id': course_id,'name': name,'time': time,'isRelated': isRelated}
        return self.call('clip.course.add',params)

    def documentUpload(self,roomid, file):
        params = {'roomid': roomid,'name': file['name']}

        retval = self.call('document.uploadurl.get',params)

        retval = eval(retval)

        if retval['code'] == MTCloud.CODE_SUCCESS and retval['data']['api']:
            file['field'] = retval['data']['field']
            return self.uploadFile(retval['data']['api'], {}, file)

        return retval


    def documentDownload(self,id):
        params = {'id': id}
        return self.call('document.downloadurl.get',params)

    def documentList(self,roomid):
        params = {'roomid': roomid}
        return self.call('document.list',params)

    def documentGet(self,id):
        params = {'id': id}
        return self.call('document.get',params)

    def departmentCreate(self,departmentName):
        params = {'departmentName': departmentName}
        return self.call('department.create',params)

    def departmentUpdate(self,departmentId,departmentName):
        params = {'departmentId': departmentId, 'departmentName': departmentName}
        return self.call('department.update',params)

    def departmentDelete(self,departmentId):
        params = {'departmentId': departmentId}
        return self.call('department.delete',params)

    def departmentGet(self,departmentId):
        params = {'departmentId': departmentId}
        return self.call('department.get',params)

    def departmentGetBatch(self,departmentIds):
        params = {'departmentIds': departmentIds}
        return self.call('department.getBatch',params)
    
    def videoGetUploadUrl(self,account,accountType,title,md5,options):
        params = {'account':account,'accountType':accountType,'title':title,'md5':md5,'options':options}
        return self.call('video.getUploadUrl',params)

    def videoGet(self,videoId, expire = 3600):
        params = {'videoId':videoId,'expire':expire}
        return self.call('video.get',params)

    def videoGetBatch(self,videoIds, expire = 3600):
        params = {'videoIds':videoIds,'expire':expire}
        return self.call('video.getBatch',params)

    def videoList(self,page, expire = 3600):
        params = {'page':page,'expire':expire}
        return self.call('video.list',params)
        
    def videoDelete(self,videoId):
        params = {'videoId':videoId}
        return self.call('video.delete',params)
        
    def videoUpload(self,filename,account,accountType,title,nickname = '', accountIntro = '',course = {}):
        retval = '';
        
        if not os.path.isfile(filename):
            return {'code':self.CODE_FAIL,'msg':'file not exists'}
        
        fileMd5 = self.fileMd5(filename)
        
        options = {'nickname':nickname,'accountIntro':accountIntro,'course':course}
        retval = self.videoGetUploadUrl(account, accountType, title, fileMd5, options);
        retval = eval(retval)
        if retval['code'] == self.CODE_SUCCESS:
            return self.uploadFile(retval['data']['uploadUrl'],{},{'file':filename,'name':'video','field':retval['data']['field']})
        elif retval['code'] == self.CODE_VIDEO_UPLOADED:
            retval['code'] = selfCODE_SUCCESS

        return retval

    def courseSignList(self,course_id,page = 1,size = 10):
        params = {'course_id': course_id,'page': page,'size': size}
        return self.call('course.sign',params)

    def courseSignDetail(self,course_id,signId,page = 1,size = 10):
        params = {'course_id': course_id,'signId':signId,'page': page,'size': size}
        return self.call('course.sign.detail',params)

    def utilsShortUrl(self,url):
        params = {'url': url}
        return self.call('utils.shorturl',params)

    def call(self,cmd,params,httpMethod='GET',file=''):

        sysParams = {
            'openID': self.openID.strip(),
            'timestamp': str(time.time()),
            'ver': self.version,
            'format': self.format,
            'cmd': cmd,
            'params': quote(json.dumps(params))
        }
        sysParams['sign'] = self.generateSign(sysParams)

        query = urlencode(sysParams)

        if httpMethod == 'POST':
            retval = self.doPost(sysParams,file)
        else:
            retval = self.doGet(query)

        if False == retval:
            self.endpoint = self.endpoint2
            if httpMethod == 'POST':
                retval = self.doPost(sysParams,file)
            else:
                retval = self.doGet(query)

        return retval

    def generateSign(self,params,salt=''):
        sortkeys = sorted(params.keys())
        keyStr = ''
        for key in sortkeys:
            keyStr += str(key) + str(params[key])

        keyStr += self.openToken.strip()
        return hashlib.md5(keyStr.encode('UTF-8')).hexdigest()

    def doGet(self,query):
        try:
            conn = httplib.HTTPConnection(self.endpoint,timeout=self.timeout)
            conn.request('GET','/portal.php?' + query, '', {'User-Agent':'MT-PYTHON-SDK'})

            resp = conn.getresponse()
            status = resp.status
            if status == 200:
                return resp.read().decode('UTF-8');
            else:
                return False
        except Exception as e:
            if self.endpoint == self.endpoint2:
                raise MTCloudException(str(e))
            else:
                return False
        finally:
            if conn:
                conn.close()

        return False

    def doPost(self, params, file = ''):
        import urllib

        if '' != file:
            return self.uploadFile('http://'+self.endpoint+'/portal.php', params, file)

        try:
            # params = urllib.urlencode({'name': 'tom', 'age': 22, '@file': open('C:\\Users\\zhang\\Desktop\\vim.jpg', 'rb')})
            params = urllib.urlencode(params)
            headers = {"Content-type": "application/x-www-form-urlencoded"
                , "Accept": "text/plain", "User-Agent":"MT-PYTHON-SDK"}

            httpClient = httplib.HTTPConnection(self.endpoint, 80, self.timeout)
            httpClient.request("POST", "/portal.php", params, headers)

            # conn = httplib.HTTPConnection(self.endpoint, timeout=self.timeout)
            # conn.request('POST', '/portal.php?' + query)

            resp = httpClient.getresponse()
            status = resp.status
            if status == 200:
                return resp.read().decode('UTF-8');
            else:
                return False
        except Exception as e:
            if self.endpoint == self.endpoint2:
                raise MTCloudException(str(e))
            else:
                return False
        finally:
            if httpClient:
                httpClient.close()

        return False

    def uploadFile(self, url, params, file):
        import urllib2

        data = []

        boundary = '----------%s' % hex(int(time.time() * 1000))
        data.append('--%s' % boundary)

        url = url.replace("\\", '')
        url = url.replace('\u0026', '&')
        url = url.replace('u0026', '&')

        for key in params:
            data.append('Content-Disposition: form-data; name="%s"\r\n' % key)
            data.append(params[key])
            data.append('--%s' % boundary)

        fr = open(file['file'], 'rb')
        data.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (file['field'], file['file'].split("/")[-1]))
        data.append("Content-Type: %s\r\n" % mimetypes.guess_type(file['file'])[0])
        data.append(fr.read())
        fr.close()
        data.append('--%s--\r\n' % boundary)
        http_body = '\r\n'.join(data)
        try:
            # buld http request
            req = urllib2.Request(url, data=http_body)
            # header
            req.add_header('Content-Type', 'multipart/form-data; boundary=%s' % boundary)

            # req.add_header('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8')
            self.timeout = 3600
            resp = urllib2.urlopen(req, timeout=self.timeout)
            retval = resp.read()
            return retval
        except Exception, e:
            return False
    def fileMd5(self,filename):
        f = open(filename,'rb')  
        obj = hashlib.md5()
        while True:
            d = f.read(8096)
            if not d:
                break
            obj.update(d)
        md5 = str(obj.hexdigest()).lower();
        f.close()
        return md5

    def generateGuestId(self):
        return uuid.uuid1()

    def base64UrlEncode(self,input):
        input = base64.b64encode(input)
        input = input.replace('=', '')
        input = input.replace('+', '-')
        input = input.replace('/', '_')

        return input

class MTCloudException(Exception):
    def __init__(self,msg):
        self.msg = msg
        super(MTCloudException, self).__init__(msg)

if __name__ == '__main__':
    pass
